package frc.robot.constants;

import static edu.wpi.first.units.Units.Volts;
import static edu.wpi.first.units.Units.Second;
import static edu.wpi.first.units.Units.Seconds;

import com.revrobotics.spark.config.SparkMaxConfig;
import com.revrobotics.spark.config.SparkBaseConfig.IdleMode;

import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine;
import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine.Config;

public class ElevatorConstants {
    public static final int kElevatorMotor1ID = 8;
    public static final int kElevatorMotor2ID = 6;

    public static final int kBottomLimitSwitchID = 0;

    // 60/11 gearing multiplied by circumference of sprocket multiplied by 2 for carriage position
    public static final double kEncoderPositionConversionFactor = 11.0/60.0 * (22.0*0.25) * 2.0;
    public static final double kEncoderVelocityConversionFactor = kEncoderPositionConversionFactor / 60;

    public static final int kCurrentLimit = 40;

    public static final double kUpControllerP = 5.6;//7; //
    public static final double kUpControllerI = 0;
    public static final double kUpControllerD = 0.28;//0.28

    public static final double kDownControllerP = 5.6;//7; //
    public static final double kDownControllerI = 0;
    public static final double kDownControllerD = 0.57;//0.175;//0.1;//0.35

    public static final double kMaintainP = 3;
     
    public static final double kAllowedError = 1;

    public static final double kFeedForwardS = (0.95 - 0.2)/2*0.8;   /* kG too high - kG too low / 2  0.95, 0.2 */
    public static final double kFeedForwardG = (0.95 + 0.2)/2;  /* kG too high + kG too low / 2 */    // calculated value 0.6
    public static final double kFeedForwardV = 0.12;   // calculated value 0.12

    public static final double kMaxVelocity = 150.0;    // 120 inches per second (COOKING) calculated max is 184 in/s
    public static final double kMaxAcceleration = 240;    // 400 inches per second^2 (also COOKING) calculated max is 600 in/s^2

    public static final double kCoralIntakePosition = 0;
    public static final double kL1Position = 0;
    public static final double kL2Position = 11;
    public static final double kL3Position = 27;
    public static final double kL4Position = 50.5;
    public static final double kL4TransitionPosition = 40.0;
    public static final double kL2AlgaePosition = 23.0;
    public static final double kL3AlgaePosition = 39.0;
    public static final double kProcessorPosition = 4.0;
    /**The position of the top of the elevator brace */
    public static final double kBracePosition = 0;
    public static final double kMaxHeight = 51.0; //actual is 51

    public static final double kVoltageLimit = 7;

    // 1, 7, 10 are the defaults for these, change as necessary
    public static final double kSysIDRampRate = .25;
    public static final double kSysIDStepVolts = 3;
    public static final double kSysIDTimeout = 10;

    public static final IdleMode kIdleMode = IdleMode.kBrake;

    // YOU SHOULDN'T NEED TO CHANGE ANYTHING BELOW THIS LINE UNLESS YOU'RE ADDING A NEW CONFIGURATION ITEM
    
    public static final SysIdRoutine.Config kSysIDConfig = new Config(
        Volts.of(kSysIDRampRate).per(Second), 
        Volts.of(kSysIDStepVolts), 
        Seconds.of(kSysIDTimeout)
    );

    public static final SparkMaxConfig motorConfig = new SparkMaxConfig();
    
    static {
        motorConfig
            .smartCurrentLimit(kCurrentLimit)
            .idleMode(kIdleMode)
            .inverted(true);
        motorConfig.encoder
            .positionConversionFactor(kEncoderPositionConversionFactor)
            .velocityConversionFactor(kEncoderVelocityConversionFactor);
    }
}
