package frc.robot.constants;

import static edu.wpi.first.units.Units.Volts;
import static edu.wpi.first.units.Units.Second;
import static edu.wpi.first.units.Units.Seconds;

import com.revrobotics.spark.config.SparkMaxConfig;
import com.revrobotics.spark.config.SparkBaseConfig.IdleMode;

import edu.wpi.first.math.util.Units;
import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine;
import edu.wpi.first.wpilibj2.command.sysid.SysIdRoutine.Config;

public class ManipulatorPivotConstants {
    public static final int kPivotMotorID = 1;

    public static final int kMotorCurrentMax = 40;

    public static final double kPivotConversion = 2 * Math.PI;

    public static final double kPivotMaxVelocity = 2 * Math.PI;

    public static final double kPositionalP = 4;
    public static final double kPositionalI = 0;
    public static final double kPositionalD = 0;
    public static final double kPositionalTolerance = Units.degreesToRadians(1.5);

    public static final double kFeedForwardS = (0.3-0.19) / 2 * 0.8; //upper: 0.3 lower: 0.19
    public static final double kFeedForwardG = (0.3+0.19) / 2; // calculated value 0.41
    public static final double kFeedForwardV = 0.68; //calculated value 0.68

    public static final double kFFGravityOffset = Units.degreesToRadians(135.0+90);

    public static final double kMaxAcceleration = Units.degreesToRadians(1000.0); // degrees per second^2 calculated max = 2100
    public static final double kMaxVelocity = Units.degreesToRadians(100.0); // degrees per second calculated max = 168

    public static final double kEncoderOffset = 0.78-0.25;

    public static final double kStartingPosition = Units.degreesToRadians(90);
    public static final double kCoralIntakePosition = Units.degreesToRadians(175.0+90);
    public static final double kL1Position = Units.degreesToRadians(0.0+90);
    public static final double kL2Position = Units.degreesToRadians(22.0+90);
    public static final double kL3Position = Units.degreesToRadians(22.0+90);
    public static final double kL4Position = Units.degreesToRadians(45.0+90);
    public static final double kL2AlgaePosition = Units.degreesToRadians(175.0+90);
    public static final double kL3AlgaePosition = Units.degreesToRadians(175.0+90);
    public static final double kProcessorPosition = Units.degreesToRadians(175.0+90);
    public static final double kNetPosition = Units.degreesToRadians(175.0+90);
    /**The closest position to the elevator brace without hitting it */
    public static final double kPivotSafeStowPosition = Units.degreesToRadians(71.0+90);
    /**The forward rotation limit of the pivot */
    public static final double kRotationLimit = Units.degreesToRadians(175.0+90);

    public static final double kSysIDRampRate = 1;
    public static final double kSysIDStepVolts = 7;
    public static final double kSysIDTimeout = 10;

    public static final IdleMode kIdleMode = IdleMode.kBrake;

    // YOU SHOULDN'T NEED TO CHANGE ANYTHING BELOW THIS LINE UNLESS YOU'RE ADDING A NEW CONFIG

    public static final SysIdRoutine.Config kSysIDConfig = new Config(
        Volts.of(kSysIDRampRate).per(Second),
        Volts.of(kSysIDStepVolts),
        Seconds.of(kSysIDTimeout)
    );

    public static final SparkMaxConfig motorConfig = new SparkMaxConfig();

    static {
        motorConfig
            .smartCurrentLimit(kMotorCurrentMax)
            .idleMode(kIdleMode)
            .inverted(true);
        motorConfig.absoluteEncoder
            .positionConversionFactor(kPivotConversion)
            .inverted(false)
            .zeroOffset(kEncoderOffset);
    }


}
